/*
 * TUX - Integrated Application Protocols Layer and Object Cache
 *
 * Copyright (C) 2000, 2001, Ingo Molnar <mingo@redhat.com>
 *
 * times.c: time conversion routines.
 *
 * Original time convserion code Copyright (C) 1999 by Arjan van de Ven
 */

/****************************************************************
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************/

#include <linux/time.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/ctype.h>


#include "times.h"

char *dayName[7] = {
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

static char *monthName[12] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

char itoa_h[60]={'0','0','0','0','0','0','0','0','0','0',
		 '1','1','1','1','1','1','1','1','1','1',
		 '2','2','2','2','2','2','2','2','2','2',
		 '3','3','3','3','3','3','3','3','3','3',
		 '4','4','4','4','4','4','4','4','4','4',
		 '5','5','5','5','5','5','5','5','5','5'};
		
char itoa_l[60]={'0','1','2','3','4','5','6','7','8','9',
		 '0','1','2','3','4','5','6','7','8','9',
		 '0','1','2','3','4','5','6','7','8','9',
		 '0','1','2','3','4','5','6','7','8','9',
		 '0','1','2','3','4','5','6','7','8','9',
		 '0','1','2','3','4','5','6','7','8','9'};

int time_unix2ls (time_t zulu, char *buf)
{
	int Y=0,M=0,D=0;
	int H=0,Min=0,S=0,WD=0;
	int I,I2;
	time_t rest, delta;

	if (zulu > xtime.tv_sec)
		zulu = xtime.tv_sec;

	I=0;
	while (I<KHTTPD_NUMYEARS)
	{
		if (TimeDays[I][0]>zulu) 
		   break;
		I++;
	}
	
	Y=--I;
	if (I<0) 
	{
		Y=0;
		goto BuildYear;
	}
	I2=0;
	while (I2<=12)
	{
		if (TimeDays[I][I2]>zulu) 
		   break;
		I2++;
	}			   
	
	M=I2-1;
	
	rest=zulu - TimeDays[Y][M];
	WD=WeekDays[Y][M];
	D=rest/86400;
	rest=rest%86400;
	WD+=D;
	WD=WD%7;
	H=rest/3600;
	rest=rest%3600;
	Min=rest/60;
	rest=rest%60;
	S=rest;
	
BuildYear:
	Y+=KHTTPD_YEAROFFSET;
	
	
	/* Format:  Day, 01 Mon 1999 01:01:01 GMT */

	delta = xtime.tv_sec - zulu;
	if (delta > 6*30*24*60)
		//               "May 23   2000"
		return sprintf( buf, "%s %02i  %04i", monthName[M], D+1, Y);
	else
		//                "May 23 10:14"
		return sprintf( buf, "%s %02i %02i:%02i",
			monthName[M], D+1, H, Min);
}

static int MonthHash[32] = {0,0,7,0,0,0,0,0,0,0,0,3,0,0,0,2,6,0,5,0,9,8,4,0,0,11,1,10,0,0,0,0};

#define is_digit(c)	((c) >= '0' && (c) <= '9')

__inline static int skip_atoi(char **s)
{
	int i=0;

	while (is_digit(**s))
		i = i*10 + *((*s)++) - '0';
	return i;
}

time_t mimetime_to_unixtime(char *Q)
{
	int Y,M,D,H,Min,S;
	unsigned int Hash;
	time_t Temp;
	char *s,**s2;
	
	s=Q;
	s2=&s;
	
	if (strlen(s)<30) return 0;
	if (s[3]!=',') return 0;
	if (s[19]!=':') return 0;
	
	s+=5; /* Skip day of week */
	D = skip_atoi(s2);  /*  Day of month */
	s++;
	Hash = (unsigned char)s[0]+(unsigned char)s[2];
	Hash = (Hash<<1) + (unsigned char)s[1];
	Hash = (Hash&63)>>1;
	M = MonthHash[Hash];
	s+=4;
	Y = skip_atoi(s2); /* Year */
	s++;
	H = skip_atoi(s2); /* Hour */
	s++;
	Min = skip_atoi(s2); /* Minutes */
	s++;
	S = skip_atoi(s2); /* Seconds */
	s++;
	if ((s[0]!='G')||(s[1]!='M')||(s[2]!='T')) 
	{	
  		return 0; /* No GMT */
  	}

	if (Y<KHTTPD_YEAROFFSET) Y = KHTTPD_YEAROFFSET;
	if (Y>KHTTPD_YEAROFFSET+9) Y = KHTTPD_YEAROFFSET+9;
	
	Temp = 	TimeDays[Y-KHTTPD_YEAROFFSET][M];
	Temp += D*86400+H*3600+Min*60+S;
	
	return Temp;  
}
